import datetime
from typing import TypeVar, List, Optional, Dict, Generic, Union, Any, Literal
from pydantic import BaseModel, Field

DataT = TypeVar('DataT')

Gender = Literal['male', 'female']
Nation = Literal['russian', 'american', 'german', 'french', 'italian', 'japanese', 'latinos', 'swedish', 'danish', 'romanian']

PunishType = Literal['kick', 'warn', 'warnoff', 'jail', 'jailoff', 'mute', 'muteoff', 'rmute', 'ban', 'banip', 'unjail',
                     'unmute', 'unrmute', 'apunish', 'unapunish']


class FastAPIErrorDetail(BaseModel):
    loc: List[str]
    message: str = Field(alias='msg')
    type: str

 
class FastApiErrorResponse(BaseModel):
    detail: Union[list[FastAPIErrorDetail], str]


class APIErrorResponse(BaseModel):
    error_code: int
    detail: str
    queue_position: Optional[int] = None


class Response(BaseModel, Generic[DataT]):
    result_data: Optional[DataT] = None
    error: Optional[Union[FastApiErrorResponse, APIErrorResponse]] = None
    success: bool = True


class MembersAPIPlayer(BaseModel):
    account_id: Optional[int]
    nickname: str
    is_online: bool
    is_leader: bool
    rank_number: int
    rank_label: Optional[str]
    ingame_id: Optional[int]
    nickname_color: Optional[int]


class MembersAPIRecord(BaseModel):
    online_players: int
    leader_nickname: Optional[str]
    modified_at: Optional[datetime.datetime]
    modified_by: Literal['system', 'admin']


class MembersFractionData(BaseModel):
    fraction_id: int
    fraction_label: str
    players: List[MembersAPIPlayer]
    total_players: int
    total_online: int
    leader_nickname: Optional[str]
    is_leader_online: bool
    online_updated_at: datetime.datetime
    members_updated_at: datetime.datetime
    online_record: MembersAPIRecord


class MembersAPIResponse(BaseModel):
    server_id: int
    server_label: str
    data: List[MembersFractionData]


class ServerStatusAPIResponse(BaseModel):
    server_id: int
    server_label: str
    server_ip: str
    server_port: int
    server_vk: Optional[str]
    server_ds: Optional[str]
    online_players: int
    max_players: int
    is_closed: bool
    main_admin_vk: Optional[str]
    deputy_main_admin_vk: Optional[str]
    updated_at: datetime.datetime


class CreatedFindTaskAPIResponse(BaseModel):
    request_id: str
    request_time: int
    queue_position: int


class PlayerInfo(BaseModel):
    nickname: str
    ingame_id: int
    nickname_color: int
    lvl: int
    ping: int


class PlayerInfoAPIResponse(BaseModel):
    server_id: int
    server_label: str
    updated_at: datetime.datetime
    data: List[PlayerInfo]


class RatingPlayerInfo(BaseModel):
    ranking: int
    nickname: str
    value: Optional[Any]
    additional_value: Optional[Any]
    az_coins: int
    family: Optional[str] = None


class RatingPlayerInfoCrossServer(BaseModel):
    ranking: int
    nickname: str
    value: Optional[Any]
    additional_value: Optional[Any]
    az_coins: int
    family: Optional[str] = None
    server_id: int
    server_label: str


class RatingAPIResponseCrossServer(BaseModel):
    server_id: int
    server_label: str
    updated_at: datetime.datetime
    data: Dict[str, list[Optional[RatingPlayerInfoCrossServer]]]


class RatingAPIResponse(BaseModel):
    server_id: int
    server_label: str
    updated_at: datetime.datetime
    data: Dict[str, list[Optional[RatingPlayerInfo]]]


class CheckRPUsernameInfo(BaseModel):
    value: Optional[str] = None
    is_existing: bool
    is_confirmed: bool
    nationalities_chart: Optional[str] = None


class CheckRPUsernameAPIResponse(BaseModel):
    first_name: CheckRPUsernameInfo
    last_name: CheckRPUsernameInfo
    nickname: str


class GhettoZonesData(BaseModel):
    grove: int
    ballas: int
    vagos: int
    rifa: int
    aztec: int
    nw: int


class ServerMapAPIResponse(BaseModel):
    image: str
    ghetto_zones: GhettoZonesData
    server_id: int
    server_label: str
    updated_at: datetime.datetime


class AuctionInfo(BaseModel):
    active: bool
    minimal_bet: int
    time_end: Optional[datetime.datetime]
    start_price: int


class Coordinates(BaseModel):
    x: float
    y: float


class EstateItemInfo(BaseModel):
    id: int
    name: Optional[str]
    auction: AuctionInfo
    coordinates: Coordinates


class EstatePlayer(BaseModel):
    houses: List[EstateItemInfo]
    businesses: List[EstateItemInfo]


class PlayerEstateAPIResponse(BaseModel):
    server_id: int
    server_label: str
    updated_at: datetime.datetime
    data: Dict[str, EstatePlayer]


class RPNickname(BaseModel):
    first_name: str
    last_name: str
    nickname: str


class GenerateRPUsernameAPIResponse(BaseModel):
    gender: Gender
    nation: Nation
    data: list[RPNickname]


class PlayerSessionInfo(BaseModel):
    login_at: datetime.datetime
    logout_at: datetime.datetime


class PlayerSessionsAPIResponse(BaseModel):
    server_id: int
    server_label: str
    nickname: str
    data: List[PlayerSessionInfo]


class PlayerGameInfo(BaseModel):
    nickname: str
    ingame_id: int
    nickname_color: int
    lvl: int
    ping: int


class PlayersAPIResponse(BaseModel):
    server_id: int
    server_label: str
    updated_at: datetime.datetime
    data: List[PlayerGameInfo]


class TokenStatCountsAPIResponse(BaseModel):
    find: Optional[int] = None
    members: Optional[int] = None
    status: Optional[int] = None
    rating: Optional[int] = None
    rpnick: Optional[int] = None
    checkrp: Optional[int] = None
    players: Optional[int] = None
    estate: Optional[int] = None
    map: Optional[int] = None
    sessions: Optional[int] = None
    generate_ss: Optional[int] = None


class TokenStatRequest(BaseModel):
    request_id: int = Field(alias='id')
    method: str
    params: dict[str, Any]
    ip_address: str
    requested_at: datetime.datetime


class TokenStatRequestsAPIResponse(BaseModel):
    data: List[TokenStatRequest]


class FindPlayerInfoAPIResponse(BaseModel):
    server_id: int
    server_label: str
    nickname: str
    health: int
    hunger: int
    max_hunger: int
    lvl: int
    gender: Gender
    played_hours: int
    law_count: int
    wanted_lvl: int
    marriage: Optional[str]
    job_label: str
    drug_addict_lvl: int
    phone_number: Optional[int]
    warns_count: int
    vip_lvl: int
    vip_label: str
    vip_expiration_date: Optional[datetime.datetime]
    have_addition_vip: bool
    addition_vip_expiration_date: Optional[datetime.datetime]
    current_xp: int
    max_xp: int
    az_coins: int
    total_money: int
    bank_balance: int
    have_bank_card: bool
    charity_money: int
    phone_balance: int
    cash: int
    deposit: int
    individual_accounts: Dict[str, Optional[int]]


class FindPlayerInfoNotFound(APIErrorResponse):
    pass


class DeputyInfo(BaseModel):
    fraction_id: int
    fraction_label: str
    nickname: str
    ingame_id: int
    phone_number: Optional[int]
    afk: int


class DeputiesAPIResponse(BaseModel):
    server_id: int
    server_label: str
    updated_at: datetime.datetime
    data: List[DeputyInfo]


class LeaderInfo(BaseModel):
    fraction_id: int
    fraction_label: str
    nickname: str
    ingame_id: int
    phone_number: Optional[int]
    afk: int


class LeadersAPIResponse(BaseModel):
    server_id: int
    server_label: str
    updated_at: datetime.datetime
    data: List[LeaderInfo]


class PunishInfo(BaseModel):
    punish_type: PunishType
    player_nickname: str
    admin_nickname: str
    reason: str
    full_string: str
    created_at: datetime.datetime
    expires_at: Optional[datetime.datetime]


class PunishesAPIResponse(BaseModel):
    server_id: int
    server_label: str
    data: List[PunishInfo]


class InterviewInfo(BaseModel):
    fraction_id: int
    fraction_label: str
    start_datetime: datetime.datetime
    place: str


class InterviewsAPIResponse(BaseModel):
    server_id: int
    server_label: str
    updated_at: datetime.datetime
    data: List[InterviewInfo]



from typing import Optional, Dict

import aiohttp
from aiohttp import FormData

from .model import Response


async def get_json(url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None) -> Response:
    async with aiohttp.ClientSession(headers=headers) as session:
        async with session.get(url, params=params) as response:
            response_json = await response.json()
            if response.ok:
                return Response(result_data=response_json)
            return Response(error=response_json, success=False)


async def post_json(url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None) -> Response:
    async with aiohttp.ClientSession(headers=headers) as session:
        async with session.post(url, params=params) as response:
            response_json = await response.json()
            if response.ok:
                return Response(result_data=response_json)
            return Response(error=response_json, success=False)


async def get_bytes(url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None,
                    post_data: FormData = None) -> Response:
    async with aiohttp.ClientSession(headers=headers) as session:

        if not post_data:
            async with session.get(url, params=params) as response:
                response_data = await response.read()
                if response.status == 200:
                    return Response(result_data=response_data)
                return Response(error=(await response.json()), success=False)

        async with session.post(url, params=params, data=post_data) as response:
            response_data = await response.read()
            if response.status == 200:
                return Response(result_data=response_data)
            return Response(error=(await response.json()), success=False)



import datetime
from io import BytesIO
from typing import List, Literal, Optional, Union

from aiohttp import FormData
from pydantic import parse_obj_as

from .api import get_json, get_bytes
from .model import (MembersAPIResponse, PlayerInfoAPIResponse,
                    ServerStatusAPIResponse, RatingAPIResponse, CheckRPUsernameAPIResponse,
                    GenerateRPUsernameAPIResponse, PlayerSessionsAPIResponse, PlayerEstateAPIResponse,
                    PlayersAPIResponse, Gender, Nation, ServerMapAPIResponse, TokenStatCountsAPIResponse,
                    TokenStatRequestsAPIResponse, FindPlayerInfoNotFound, FindPlayerInfoAPIResponse,
                    RatingAPIResponseCrossServer, DeputiesAPIResponse, LeadersAPIResponse, PunishesAPIResponse,
                    PunishType, InterviewsAPIResponse)


class VprikolAPI:
    def __init__(self, token: str, base_url: str = 'https://api.szx.su/'):
        if not token.startswith('eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.'):
            raise Exception('Вы указали некорректный авторизационный токен.')
        self.headers = {'Authorization': f'Bearer {token}'}
        self.base_url = base_url

    async def get_members(self, server_id: int,
                          fraction_ids: Optional[Union[int, List[int]]] = None) -> MembersAPIResponse:
        params = {'server_id': server_id}
        if fraction_ids:
            params['fraction_ids'] = fraction_ids
        result = await get_json(url=f'{self.base_url}members', headers=self.headers, params=params)

        if not result.success:
            raise Exception(result.error)

        return MembersAPIResponse(**result.result_data)

    async def get_player_information(self, server_id: int, nickname: Optional[str] = None) -> Union[PlayerInfoAPIResponse]:
        params = {'server_id': server_id}
        if nickname:
            params['nickname'] = nickname
        result = await get_json(url=f'{self.base_url}players', headers=self.headers, params=params)

        if not result.success:
            raise Exception(result.error)

        return PlayerInfoAPIResponse(**result.result_data)

    async def get_server_status(self, server_id: Optional[int] = None) -> List[ServerStatusAPIResponse]:
        params = {'server_id': server_id} if server_id else None
        result = await get_json(url=f'{self.base_url}status', headers=self.headers, params=params)

        if not result.success:
            raise Exception(result.error)

        return parse_obj_as(List[ServerStatusAPIResponse], result.result_data)

    async def get_rating(self, server_id: int, rating_type: Literal["advocates", "combine_operators", "bus_drivers",
                                                                    "tractor_drivers", "catchers", "collectors",
                                                                    "corn_pilots", "crypto_asc", "crypto_btc",
                                                                    "electric_train_drivers", "lvl_families",
                                                                    "lvl_players", "mechanics", "richest", "outbids",
                                                                    "pilots", "sellers", "taxi_drivers", "tram_drivers",
                                                                    "truckers", "cladmens"] = None) -> RatingAPIResponse | RatingAPIResponseCrossServer:
        params = {'rating_type': rating_type, 'server_id': server_id} if rating_type else {'server_id': server_id}
        result = await get_json(url=f'{self.base_url}rating', headers=self.headers, params=params)

        if not result.success:
            raise Exception(result.error)
        if server_id:
            return RatingAPIResponse(**result.result_data)
        return RatingAPIResponseCrossServer(**result.result_data)

    async def check_rp_nickname(self, first_name: Optional[str] = None,
                                last_name: Optional[str] = None) -> CheckRPUsernameAPIResponse:
        params = {}
        if first_name:
            params['first_name'] = first_name
        if last_name:
            params['last_name'] = last_name
        result = await get_json(url=f'{self.base_url}checkrp', headers=self.headers, params=params)

        if not result.success:
            raise Exception(result.error)

        return CheckRPUsernameAPIResponse(**result.result_data)

    async def generate_rp_nickname(self, gender: Gender = 'male', nation: Nation = 'american',
                                   count: int = 1) -> GenerateRPUsernameAPIResponse:
        result = await get_json(url=f'{self.base_url}rpnick', headers=self.headers,
                                params={'gender': gender, 'nation': nation, 'count': count})

        if not result.success:
            raise Exception(result.error)

        return GenerateRPUsernameAPIResponse(**result.result_data)

    async def get_player_sessions(self, server_id: int, nickname: str, count: int = 1000, offset: int = 0,
                                  start_datetime: Optional[datetime.datetime] = None,
                                  end_datetime: Optional[datetime.datetime] = None) -> PlayerSessionsAPIResponse:
        params = {'nickname': nickname, 'count': count, 'offset': offset, 'server_id': server_id}
        if start_datetime:
            params['start_datetime'] = start_datetime.isoformat() + '+03:00'
        if end_datetime:
            params['end_datetime'] = end_datetime.isoformat() + '+03:00'
        result = await get_json(url=f'{self.base_url}sessions', headers=self.headers, params=params)

        if not result.success:
            raise Exception(result.error)

        return PlayerSessionsAPIResponse(**result.result_data)

    async def get_server_map(self, server_id: int, only_ghetto: bool = False) -> ServerMapAPIResponse:
        result = await get_json(url=f'{self.base_url}map', headers=self.headers,
                                params={'server_id': server_id, 'only_ghetto': int(only_ghetto)})

        if not result.success:
            raise Exception(result.error)

        return ServerMapAPIResponse(**result.result_data)

    async def get_estate(self, server_id: int, nickname: Optional[str] = None) -> PlayerEstateAPIResponse:
        params = {'server_id': server_id}
        if nickname:
            params['nickname'] = nickname
        result = await get_json(url=f'{self.base_url}estate', headers=self.headers, params=params)

        if not result.success:
            raise Exception(result.error)

        return PlayerEstateAPIResponse(**result.result_data)

    async def get_players(self, server_id: int, nicknames: Optional[List[Union[str, int]]] = None) -> PlayersAPIResponse:
        params = {'server_id': server_id}
        if nicknames:
            params['nicknames'] = nicknames
        result = await get_json(url=f'{self.base_url}players', headers=self.headers, params=params)

        if not result.success:
            raise Exception(result.error)

        return PlayersAPIResponse(**result.result_data)

    async def generate_ss(self, commands: list, screen: Union[bytes, BytesIO], font: str = '/fonts/arialbd.ttf',
                          text_top: bool = True) -> bytes:

        if isinstance(screen, bytes):
            screen = BytesIO(screen)

        data = FormData()
        data.add_field('screen', screen, filename='screen.png', content_type='application/octet-stream')

        result = await get_bytes(url=f'{self.base_url}generate_ss', headers=self.headers,
                                 params={'commands': commands, 'font': font, 'text_top': int(text_top)}, post_data=data)

        if not result.success:
            raise Exception(result.error)

        return result.result_data

    async def get_token_stat(self, methods: Optional[List[Union[str, int]]] = None,
                             start_datetime: Optional[datetime.datetime] = None,
                             end_datetime: Optional[datetime.datetime] = None,
                             response_type: Literal['counts', 'requests'] = 'counts',
                             requests_limit: int = 1000, requests_offset: int = 0) -> TokenStatCountsAPIResponse | TokenStatRequestsAPIResponse:
        params = {'response_type': response_type, 'requests_limit': requests_limit, 'requests_offset': requests_offset}
        if start_datetime:
            params['start_datetime'] = start_datetime.isoformat() + '+03:00'
        if end_datetime:
            params['end_datetime'] = end_datetime.isoformat() + '+03:00'
        if methods:
            params['methods'] = methods
        result = await get_json(url=f'{self.base_url}stat', headers=self.headers, params=params)

        if not result.success:
            raise Exception(result.error)

        if response_type == 'counts':
            return TokenStatCountsAPIResponse(**result.result_data)
        elif response_type == 'requests':
            return TokenStatRequestsAPIResponse(**result.result_data)

    async def find_player(self, server_id: int, nickname: str) -> Union[FindPlayerInfoAPIResponse, FindPlayerInfoNotFound]:
        result = await get_json(url=f'{self.base_url}find', headers=self.headers,
                                params={'server_id': server_id, 'nickname': nickname})

        if not result.success:
            raise Exception(result.error)

        if result.error and result.error.error_code == 422:
            return FindPlayerInfoNotFound(**result.error.dict())

        if result.error and result.error.error_code in [500, 502, 503]:
            raise Exception(result.error.detail)

        return FindPlayerInfoAPIResponse(**result.result_data)

    async def get_deputies(self, server_id: int) -> DeputiesAPIResponse:
        result = await get_json(url=f'{self.base_url}deputies', headers=self.headers, params={'server_id': server_id})

        if not result.success:
            raise Exception(result.error)

        return DeputiesAPIResponse(**result.result_data)

    async def get_leaders(self, server_id: int) -> LeadersAPIResponse:
        result = await get_json(url=f'{self.base_url}leaders', headers=self.headers, params={'server_id': server_id})

        if not result.success:
            raise Exception(result.error)

        return LeadersAPIResponse(**result.result_data)

    async def get_punishes(self, server_id: int,
                           player_nickname: Optional[str] = None, admin_nickname: Optional[str] = None,
                           start_datetime: Optional[datetime.datetime] = None,
                           end_datetime: Optional[datetime.datetime] = None,
                           punish_type: Optional[PunishType] = None,
                           count: int = 1000, offset: int = 0) -> PunishesAPIResponse:
        params = {'server_id': server_id, 'count': count, 'offset': offset}
        if start_datetime:
            params['start_datetime'] = start_datetime.isoformat() + '+03:00'
        if end_datetime:
            params['end_datetime'] = end_datetime.isoformat() + '+03:00'
        if player_nickname:
            params['player_nickname'] = player_nickname
        if admin_nickname:
            params['admin_nickname'] = admin_nickname
        if punish_type:
            params['punish_type'] = punish_type
        result = await get_json(url=f'{self.base_url}punishes', headers=self.headers, params=params)

        if not result.success:
            raise Exception(result.error)

        return PunishesAPIResponse(**result.result_data)

    async def get_active_interviews(self, server_id: int) -> InterviewsAPIResponse:
        result = await get_json(url=f'{self.base_url}interviews', headers=self.headers, params={'server_id': server_id})

        if not result.success:
            raise Exception(result.error)

        return InterviewsAPIResponse(**result.result_data)
